using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman52 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.2.42 kappa*(T2-T1)*A/d"; } }

    protected override string TargetVariable { get { return "Pwr"; } }
    protected override string[] VariableNames { get { return new[] {"kappa", "T1", "T2", "A", "d", "Pwr"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"kappa", "T1", "T2", "A", "d"}; } }

    public int Seed { get; private set; }

    public Feynman52() : this((int) DateTime.Now.Ticks) { }

    public Feynman52(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var kappa = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T1    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T2    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var A     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Pwr = new List<double>();

      data.Add(kappa);
      data.Add(T1);
      data.Add(T2);
      data.Add(A);
      data.Add(d);
      data.Add(Pwr);

      for (var i = 0; i < kappa.Count; i++) {
        var res = kappa[i] * (T2[i] - T1[i]) * A[i] / d[i];
        Pwr.Add(res);
      }

      return data;
    }
  }
}