using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman53 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.3.24 Pwr/(4*pi*r**2)"; } }

    protected override string TargetVariable { get { return "flux"; } }
    protected override string[] VariableNames { get { return new[] {"Pwr", "r", "flux"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"Pwr", "r"}; } }

    public int Seed { get; private set; }

    public Feynman53() : this((int) DateTime.Now.Ticks) { }

    public Feynman53(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var Pwr  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var flux = new List<double>();

      data.Add(Pwr);
      data.Add(r);
      data.Add(flux);

      for (var i = 0; i < Pwr.Count; i++) {
        var res = Pwr[i] / (4 * Math.PI * Math.Pow(r[i], 2));
        flux.Add(res);
      }

      return data;
    }
  }
}