using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman54 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.4.23 q/(4*pi*epsilon*r)"; } }

    protected override string TargetVariable { get { return "Volt"; } }
    protected override string[] VariableNames { get { return new[] {"q", "epsilon", "r", "Volt"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "epsilon", "r"}; } }

    public int Seed { get; private set; }

    public Feynman54() : this((int) DateTime.Now.Ticks) { }

    public Feynman54(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Volt = new List<double>();

      data.Add(q);
      data.Add(epsilon);
      data.Add(r);
      data.Add(Volt);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] / (4 * Math.PI * epsilon[i] * r[i]);
        Volt.Add(res);
      }

      return data;
    }
  }
}