using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman55 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.6.11 1/(4*pi*epsilon)*p_d*cos(theta)/r**2"; } }

    protected override string TargetVariable { get { return "Volt"; } }
    protected override string[] VariableNames { get { return new[] {"epsilon", "p_d", "theta", "r", "Volt"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"epsilon", "p_d", "theta", "r"}; } }

    public int Seed { get; private set; }

    public Feynman55() : this((int) DateTime.Now.Ticks) { }

    public Feynman55(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var p_d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var theta   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var Volt = new List<double>();

      data.Add(epsilon);
      data.Add(p_d);
      data.Add(theta);
      data.Add(r);
      data.Add(Volt);

      for (var i = 0; i < epsilon.Count; i++) {
        var res = 1.0 / (4 * Math.PI * epsilon[i]) * p_d[i] * Math.Cos(theta[i]) / Math.Pow(r[i], 2);
        Volt.Add(res);
      }

      return data;
    }
  }
}