using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman56 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.6.15a p_d/(4*pi*epsilon)*3*z/r**5*sqrt(x**2+y**2)"; } }

    protected override string TargetVariable { get { return "Ef"; } }
    protected override string[] VariableNames { get { return new[] {"epsilon", "p_d", "r", "x", "y", "z", "Ef"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"epsilon", "p_d", "r", "x", "y", "z"}; } }

    public int Seed { get; private set; }

    public Feynman56() : this((int) DateTime.Now.Ticks) { }

    public Feynman56(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var p_d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var x       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var y       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var z       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var Ef = new List<double>();

      data.Add(epsilon);
      data.Add(p_d);
      data.Add(r);
      data.Add(x);
      data.Add(y);
      data.Add(z);
      data.Add(Ef);

      for (var i = 0; i < epsilon.Count; i++) {
        var res = p_d[i] / (4 * Math.PI * epsilon[i]) * 3 * z[i] / Math.Pow(r[i], 2) * Math.Sqrt(Math.Pow(x[i], 2) + Math.Pow(y[i], 2));
        Ef.Add(res);
      }

      return data;
    }
  }
}