using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman57 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.6.15b p_d/(4*pi*epsilon)*3*cos(theta)*sin(theta)/r**3"; } }

    protected override string TargetVariable { get { return "Ef"; } }
    protected override string[] VariableNames { get { return new[] {"epsilon", "p_d", "theta", "r", "Ef"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"epsilon", "p_d", "theta", "r"}; } }

    public int Seed { get; private set; }

    public Feynman57() : this((int) DateTime.Now.Ticks) { }

    public Feynman57(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var p_d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var theta   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var Ef = new List<double>();

      data.Add(epsilon);
      data.Add(p_d);
      data.Add(theta);
      data.Add(r);
      data.Add(Ef);

      for (var i = 0; i < epsilon.Count; i++) {
        var res = p_d[i] / (4 * Math.PI * epsilon[i]) * 3 * Math.Cos(theta[i]) * Math.Sin(theta[i]) / Math.Pow(r[i], 3);
        Ef.Add(res);
      }

      return data;
    }
  }
}