using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman58 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.8.7 3/5*q**2/(4*pi*epsilon*d)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"q", "epsilon", "d", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "epsilon", "d"}; } }

    public int Seed { get; private set; }

    public Feynman58() : this((int) DateTime.Now.Ticks) { }

    public Feynman58(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(q);
      data.Add(epsilon);
      data.Add(d);
      data.Add(E_n);

      for (var i = 0; i < q.Count; i++) {
        var res = 3.0 / 5 * Math.Pow(q[i], 2) / (4 * Math.PI * epsilon[i] * d[i]);
        E_n.Add(res);
      }

      return data;
    }
  }
}