using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman60 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.10.9 sigma_den/epsilon*1/(1+chi)"; } }

    protected override string TargetVariable { get { return "Ef"; } }
    protected override string[] VariableNames { get { return new[] {"sigma_den", "epsilon", "chi", "Ef"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"sigma_den", "epsilon", "chi"}; } }

    public int Seed { get; private set; }

    public Feynman60() : this((int) DateTime.Now.Ticks) { }

    public Feynman60(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data      = new List<List<double>>();
      var sigma_den = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var chi       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Ef = new List<double>();

      data.Add(sigma_den);
      data.Add(epsilon);
      data.Add(chi);
      data.Add(Ef);

      for (var i = 0; i < sigma_den.Count; i++) {
        var res = sigma_den[i] / epsilon[i] * 1 / (1 + chi[i]);
        Ef.Add(res);
      }

      return data;
    }
  }
}