using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman63 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.11.20 n_rho*p_d**2*Ef/(3*kb*T)"; } }

    protected override string TargetVariable { get { return "Pol"; } }
    protected override string[] VariableNames { get { return new[] {"n_rho", "p_d", "Ef", "kb", "T", "Pol"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n_rho", "p_d", "Ef", "kb", "T"}; } }

    public int Seed { get; private set; }

    public Feynman63() : this((int) DateTime.Now.Ticks) { }

    public Feynman63(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var n_rho = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var p_d   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Pol = new List<double>();

      data.Add(n_rho);
      data.Add(p_d);
      data.Add(Ef);
      data.Add(kb);
      data.Add(T);
      data.Add(Pol);

      for (var i = 0; i < n_rho.Count; i++) {
        var res = n_rho[i] * Math.Pow(p_d[i], 2) * Ef[i] / (3 * kb[i] * T[i]);
        Pol.Add(res);
      }

      return data;
    }
  }
}