using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman64 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.11.27 n*alpha/(1-(n*alpha/3))*epsilon*Ef"; } }

    protected override string TargetVariable { get { return "Pol"; } }
    protected override string[] VariableNames { get { return new[] {"n", "alpha", "epsilon", "Ef", "Pol"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "alpha", "epsilon", "Ef"}; } }

    public int Seed { get; private set; }

    public Feynman64() : this((int) DateTime.Now.Ticks) { }

    public Feynman64(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var n       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 1).ToList();
      var alpha   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 1).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var Ef      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();

      var Pol = new List<double>();

      data.Add(n);
      data.Add(alpha);
      data.Add(epsilon);
      data.Add(Ef);
      data.Add(Pol);

      for (var i = 0; i < n.Count; i++) {
        var res = n[i] * alpha[i] / (1 - n[i] * alpha[i] / 3) * epsilon[i] * Ef[i];
        Pol.Add(res);
      }

      return data;
    }
  }
}