using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman65 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.11.28 1+n*alpha/(1-(n*alpha/3))"; } }

    protected override string TargetVariable { get { return "theta"; } }
    protected override string[] VariableNames { get { return new[] {"n", "alpha", "theta"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "alpha"}; } }

    public int Seed { get; private set; }

    public Feynman65() : this((int) DateTime.Now.Ticks) { }

    public Feynman65(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var n     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 1).ToList();
      var alpha = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 1).ToList();

      var theta = new List<double>();

      data.Add(n);
      data.Add(alpha);
      data.Add(theta);

      for (var i = 0; i < n.Count; i++) {
        var res = 1 + n[i] * alpha[i] / (1 - n[i] * alpha[i] / 3);
        theta.Add(res);
      }

      return data;
    }
  }
}