using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman68 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.13.34 rho_c_0*v/sqrt(1-v**2/c**2)"; } }

    protected override string TargetVariable { get { return "j"; } }
    protected override string[] VariableNames { get { return new[] {"rho_c_0", "v", "c", "j"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"rho_c_0", "v", "c"}; } }

    public int Seed { get; private set; }

    public Feynman68() : this((int) DateTime.Now.Ticks) { }

    public Feynman68(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var rho_c_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();

      var j = new List<double>();

      data.Add(rho_c_0);
      data.Add(v);
      data.Add(c);
      data.Add(j);

      for (var i = 0; i < rho_c_0.Count; i++) {
        var res = rho_c_0[i] * v[i] / Math.Sqrt(1 - Math.Pow(v[i], 2) / Math.Pow(c[i], 2));
        j.Add(res);
      }

      return data;
    }
  }
}