using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman70 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.15.5 -p_d*Ef*cos(theta)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"p_d", "Ef", "theta", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"p_d", "Ef", "theta"}; } }

    public int Seed { get; private set; }

    public Feynman70() : this((int) DateTime.Now.Ticks) { }

    public Feynman70(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var p_d   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(p_d);
      data.Add(Ef);
      data.Add(theta);
      data.Add(E_n);

      for (var i = 0; i < p_d.Count; i++) {
        var res = -p_d[i] * Ef[i] * Math.Cos(theta[i]);
        E_n.Add(res);
      }

      return data;
    }
  }
}