using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman71 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.21.32 q/(4*pi*epsilon*r*(1-v/c))"; } }

    protected override string TargetVariable { get { return "Volt"; } }
    protected override string[] VariableNames { get { return new[] {"q", "epsilon", "r", "v", "c", "Volt"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "epsilon", "r", "v", "c"}; } }

    public int Seed { get; private set; }

    public Feynman71() : this((int) DateTime.Now.Ticks) { }

    public Feynman71(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();

      var Volt = new List<double>();

      data.Add(q);
      data.Add(epsilon);
      data.Add(r);
      data.Add(v);
      data.Add(c);
      data.Add(Volt);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] / (4 * Math.PI * epsilon[i] * r[i] * (1 - v[i] / c[i]));
        Volt.Add(res);
      }

      return data;
    }
  }
}