using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman72 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.24.17 sqrt(omega**2/c**2-pi**2/d**2)"; } }

    protected override string TargetVariable { get { return "k"; } }
    protected override string[] VariableNames { get { return new[] {"omega", "c", "d", "k"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"omega", "c", "d"}; } }

    public int Seed { get; private set; }

    public Feynman72() : this((int) DateTime.Now.Ticks) { }

    public Feynman72(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var omega = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 4, 6).ToList();
      var c     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 2, 4).ToList();

      var k = new List<double>();

      data.Add(omega);
      data.Add(c);
      data.Add(d);
      data.Add(k);

      for (var i = 0; i < omega.Count; i++) {
        var res = Math.Sqrt(Math.Pow(omega[i], 2) / Math.Pow(c[i], 2) - Math.Pow(Math.PI, 2) / Math.Pow(d[i], 2));
        k.Add(res);
      }

      return data;
    }
  }
}