using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman74 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.27.18 epsilon*Ef**2"; } }

    protected override string TargetVariable { get { return "E_den"; } }
    protected override string[] VariableNames { get { return new[] {"epsilon", "Ef", "E_den"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"epsilon", "Ef"}; } }

    public int Seed { get; private set; }

    public Feynman74() : this((int) DateTime.Now.Ticks) { }

    public Feynman74(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_den = new List<double>();

      data.Add(epsilon);
      data.Add(Ef);
      data.Add(E_den);

      for (var i = 0; i < epsilon.Count; i++) {
        var res = epsilon[i] * Math.Pow(Ef[i], 2);
        E_den.Add(res);
      }

      return data;
    }
  }
}