using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman75 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.34.2a q*v/(2*pi*r)"; } }

    protected override string TargetVariable { get { return "I"; } }
    protected override string[] VariableNames { get { return new[] {"q", "v", "r", "I"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "v", "r"}; } }

    public int Seed { get; private set; }

    public Feynman75() : this((int) DateTime.Now.Ticks) { }

    public Feynman75(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var I = new List<double>();

      data.Add(q);
      data.Add(v);
      data.Add(r);
      data.Add(I);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] * v[i] / (2 * Math.PI * r[i]);
        I.Add(res);
      }

      return data;
    }
  }
}