using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman76 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.34.2 q*v*r/2"; } }

    protected override string TargetVariable { get { return "mom"; } }
    protected override string[] VariableNames { get { return new[] {"q", "v", "r", "mom"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "v", "r"}; } }

    public int Seed { get; private set; }

    public Feynman76() : this((int) DateTime.Now.Ticks) { }

    public Feynman76(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var mom = new List<double>();

      data.Add(q);
      data.Add(v);
      data.Add(r);
      data.Add(mom);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] * v[i] * r[i] / 2;
        mom.Add(res);
      }

      return data;
    }
  }
}