using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman77 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.34.11 g_*q*B/(2*m)"; } }

    protected override string TargetVariable { get { return "omega"; } }
    protected override string[] VariableNames { get { return new[] {"g_", "q", "B", "m", "omega"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"g_", "q", "B", "m"}; } }

    public int Seed { get; private set; }

    public Feynman77() : this((int) DateTime.Now.Ticks) { }

    public Feynman77(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var g_   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var omega = new List<double>();

      data.Add(g_);
      data.Add(q);
      data.Add(B);
      data.Add(m);
      data.Add(omega);

      for (var i = 0; i < g_.Count; i++) {
        var res = g_[i] * q[i] * B[i] / (2 * m[i]);
        omega.Add(res);
      }

      return data;
    }
  }
}