using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman78 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.34.29a q*h/(4*pi*m)"; } }

    protected override string TargetVariable { get { return "mom"; } }
    protected override string[] VariableNames { get { return new[] {"q", "h", "m", "mom"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "h", "m"}; } }

    public int Seed { get; private set; }

    public Feynman78() : this((int) DateTime.Now.Ticks) { }

    public Feynman78(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var mom = new List<double>();

      data.Add(q);
      data.Add(h);
      data.Add(m);
      data.Add(mom);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] * h[i] / (4 * Math.PI * m[i]);
        mom.Add(res);
      }

      return data;
    }
  }
}