using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman80 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.35.18 n_0/(exp(mom*B/(kb*T))+exp(-mom*B/(kb*T)))"; } }

    protected override string TargetVariable { get { return "n"; } }
    protected override string[] VariableNames { get { return new[] {"n_0", "kb", "T", "mom", "B", "n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n_0", "kb", "T", "mom", "B"}; } }

    public int Seed { get; private set; }

    public Feynman80() : this((int) DateTime.Now.Ticks) { }

    public Feynman80(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var n_0  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var mom  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var B    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var n = new List<double>();

      data.Add(n_0);
      data.Add(kb);
      data.Add(T);
      data.Add(mom);
      data.Add(B);
      data.Add(n);

      for (var i = 0; i < n_0.Count; i++) {
        var res = n_0[i] / (Math.Exp(mom[i] * B[i] / (kb[i] * T[i])) + Math.Exp(-mom[i] * B[i] / (kb[i] * T[i])));
        n.Add(res);
      }

      return data;
    }
  }
}