using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman81 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.35.21 n_rho*mom*tanh(mom*B/(kb*T))"; } }

    protected override string TargetVariable { get { return "M"; } }
    protected override string[] VariableNames { get { return new[] {"n_rho", "mom", "B", "kb", "T", "M"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n_rho", "mom", "B", "kb", "T"}; } }

    public int Seed { get; private set; }

    public Feynman81() : this((int) DateTime.Now.Ticks) { }

    public Feynman81(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var n_rho = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var mom   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var M = new List<double>();

      data.Add(n_rho);
      data.Add(mom);
      data.Add(B);
      data.Add(kb);
      data.Add(T);
      data.Add(M);

      for (var i = 0; i < n_rho.Count; i++) {
        var res = n_rho[i] * mom[i] * Math.Tanh(mom[i] * B[i] / (kb[i] * T[i]));
        M.Add(res);
      }

      return data;
    }
  }
}