using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman82 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.36.38 mom*H/(kb*T)+(mom*alpha)/(epsilon*c**2*kb*T)*M"; } }

    protected override string TargetVariable { get { return "f"; } }

    protected override string[] VariableNames {
      get { return new[] {"mom", "H", "kb", "T", "alpha", "epsilon", "c", "M", "f"}; }
    }

    protected override string[] AllowedInputVariables {
      get { return new[] {"mom", "H", "kb", "T", "alpha", "epsilon", "c", "M"}; }
    }

    public int Seed { get; private set; }

    public Feynman82() : this((int) DateTime.Now.Ticks) { }

    public Feynman82(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var mom     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var H       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var kb      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var T       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var alpha   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var c       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var M       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var f = new List<double>();

      data.Add(mom);
      data.Add(H);
      data.Add(kb);
      data.Add(T);
      data.Add(alpha);
      data.Add(epsilon);
      data.Add(c);
      data.Add(M);
      data.Add(f);

      for (var i = 0; i < mom.Count; i++) {
        var res = mom[i] * H[i] / (kb[i] * T[i]) + mom[i] * alpha[i] / (epsilon[i] * Math.Pow(c[i],2) * kb[i] * T[i]) *M[i];
        f.Add(res);
      }

      return data;
    }
  }
}