using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman83 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.37.1 mom*(1+chi)*B"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"mom", "B", "chi", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mom", "B", "chi"}; } }

    public int Seed { get; private set; }

    public Feynman83() : this((int) DateTime.Now.Ticks) { }

    public Feynman83(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mom  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var chi  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(mom);
      data.Add(B);
      data.Add(chi);
      data.Add(E_n);

      for (var i = 0; i < mom.Count; i++) {
        var res = mom[i] * (1 + chi[i]) * B[i];
        E_n.Add(res);
      }

      return data;
    }
  }
}