using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman84 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.38.3 Y*A*x/d"; } }

    protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new[] {"Y", "A", "d", "x", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"Y", "A", "d", "x"}; } }

    public int Seed { get; private set; }

    public Feynman84() : this((int) DateTime.Now.Ticks) { }

    public Feynman84(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var Y    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var A    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var x    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var F = new List<double>();

      data.Add(Y);
      data.Add(A);
      data.Add(d);
      data.Add(x);
      data.Add(F);

      for (var i = 0; i < Y.Count; i++) {
        var res = Y[i] * A[i] * x[i] / d[i];
        F.Add(res);
      }

      return data;
    }
  }
}