using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman85 : FeynmanDescriptor {
    public override string Name { get { return "Feynman II.38.14 Y/(2*(1+sigma))"; } }

    protected override string TargetVariable { get { return "mu_S"; } }
    protected override string[] VariableNames { get { return new[] {"Y", "sigma", "mu_S"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"Y", "sigma"}; } }

    public int Seed { get; private set; }

    public Feynman85() : this((int) DateTime.Now.Ticks) { }

    public Feynman85(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var Y     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var sigma = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var mu_S = new List<double>();

      data.Add(Y);
      data.Add(sigma);
      data.Add(mu_S);

      for (var i = 0; i < Y.Count; i++) {
        var res = Y[i] / (2 * (1 + sigma[i]));
        mu_S.Add(res);
      }

      return data;
    }
  }
}